package cn.ibizlab.core.disk.extensions.service;

import cn.ibizlab.core.disk.domain.SDFile;
import cn.ibizlab.core.disk.service.ISDFileService;
import cn.ibizlab.util.domain.FileItem;
import cn.ibizlab.util.errors.InternalServerErrorException;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Primary;
import org.springframework.stereotype.Service;
import org.springframework.util.DigestUtils;
import org.springframework.util.FileCopyUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.text.SimpleDateFormat;
import java.util.Date;

@Primary
@Slf4j
@Service
public class DiskCoreService {

    @Value("${ibiz.filePath:/app/file/}")
    private String fileRoot;

    private SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyMMdd");

    @Autowired
    private ISDFileService sdFileService;

    public FileItem saveFile(MultipartFile multipartFile) {
        return saveFile("",multipartFile);
    }

    public FileItem saveFile(String folder,MultipartFile multipartFile) {
        return saveFile(folder,"",multipartFile);
    }

    public FileItem saveFile(String folder,String fileId,MultipartFile multipartFile) {
        SDFile sdFile = new SDFile();
        if(!StringUtils.isEmpty(fileId))
            sdFile.setId(fileId);
        if(!StringUtils.isEmpty(folder))
            sdFile.setId(folder);
        return saveFile(sdFile,multipartFile);
    }

    public FileItem saveFile(SDFile sdFile,MultipartFile multipartFile) {
        FileItem item=null;
        try {
            String fileName = multipartFile.getOriginalFilename();
            sdFile.setName(fileName);
            String extension="."+getExtensionName(fileName);
            sdFile.setExtension(extension);
            String digestCode = DigestUtils.md5DigestAsHex(multipartFile.getInputStream());
            sdFile.setDigestCode(digestCode);
            String fileId = sdFile.getId();
            if(StringUtils.isEmpty(fileId))
            {
                fileId = simpleDateFormat.format(new Date()).concat("-").concat(digestCode);
                sdFile.setId(fileId);
            }
            String folder = sdFile.getFolder();
            if(StringUtils.isEmpty(folder))
            {
                folder = "ibizutil";
                sdFile.setFolder(folder);
            }
            String filePath =  sdFile.getFilePath();
            if(StringUtils.isEmpty(filePath)) {
                filePath = folder.concat(File.separator).concat(fileId).concat(File.separator).concat(fileName);
                sdFile.setFilePath(filePath);
            }
            int size = (int)multipartFile.getSize();
            sdFile.setFileSize(size);
            String fileFullPath = this.fileRoot.concat(filePath);
            fileFullPath = fileFullPath.replace("\\",File.separator).replace("/",File.separator);
            File file = new File(fileFullPath);
            File parent = new File(file.getParent());
            if(!parent.exists())
                parent.mkdirs();
            FileCopyUtils.copy(multipartFile.getInputStream(),Files.newOutputStream(file.toPath()));
            item=new FileItem(fileId,fileName,fileId,fileName,size,extension);
            sdFileService.save(sdFile);
        } catch (IOException e) {
            throw new InternalServerErrorException("文件上传失败");
        }
        return item;
    }

    public File getFile(String fileId) {
        return getFile("",fileId);
    }

    public File getFile(String folder,String fileId) {
        if(StringUtils.isEmpty(folder))
            folder = "ibizutil";
        String dirpath = this.fileRoot.concat(folder).concat(File.separator).concat(fileId);
        File parent = new File(dirpath);
        if (parent.exists() && parent.isDirectory() && parent.listFiles().length > 0) {
            return parent.listFiles()[0];
        }
        return getFile(sdFileService.get(fileId));
    }

    public File getFile(SDFile sdFile) {
        if(!StringUtils.isEmpty(sdFile.getFilePath())){
            String fileFullPath = this.fileRoot.concat(sdFile.getFilePath());
            fileFullPath = fileFullPath.replace("\\",File.separator).replace("/",File.separator);
            File file = new File(fileFullPath);
            if(file.exists())
                return file;
        }
        throw new InternalServerErrorException("文件未找到");
    }

    /**
     * 获取文件扩展名
     * @param fileName
     * @return
     */
    public static String getExtensionName(String fileName) {
        if ((fileName != null) && (fileName.length() > 0)) {
            int dot = fileName.lastIndexOf('.');
            if ((dot >-1) && (dot < (fileName.length() - 1))) {
                return fileName.substring(dot + 1);
            }
        }
        return fileName;
    }

}